From 0d835519ec5e3feeb00ee6182563c2fbb798f328 Mon Sep 17 00:00:00 2001
From: Tomas Janousek <tomi@nomi.cz>
Date: Thu, 1 Mar 2007 23:18:18 +0100
Subject: [PATCH] Move windows out of inactive heads upon layout change.

---
 src/Screen.cc |   25 +++++++++++++++++++++++++
 src/Screen.hh |    1 +
 2 files changed, 26 insertions(+), 0 deletions(-)

diff --git a/src/Screen.cc b/src/Screen.cc
index d992306..a78c02c 100644
--- a/src/Screen.cc
+++ b/src/Screen.cc
@@ -2144,6 +2144,9 @@ void BScreen::updateSize() {
     // send resize notify
     m_resize_sig.notify();
     m_workspace_area_sig.notify();
+
+    // move windows out of inactive heads
+    clearHeads();
 }
 
 
@@ -2238,6 +2241,28 @@ void BScreen::initXinerama() {
 
 }
 
+/* Move windows out of inactive heads */
+void BScreen::clearHeads() {
+    if (!hasXinerama()) return;
+
+    for (Workspaces::iterator i = m_workspaces_list.begin();
+	    i != m_workspaces_list.end(); i++) {
+	for (Workspace::Windows::iterator win = (*i)->windowList().begin();
+		win != (*i)->windowList().end(); win++) {
+	    if (getHead((*win)->fbWindow()) == 0) {
+		int place_x, place_y;
+		// we ignore the return value,
+		// the screen placement strategy is guaranteed to succeed.
+		placementStrategy().placeWindow((*i)->windowList(),
+			**win,
+			place_x, place_y);
+
+		(*win)->moveResize(place_x, place_y, (*win)->width(), (*win)->height());
+	    }
+	}
+    }
+}
+
 int BScreen::getHead(int x, int y) const {
     if (!hasXinerama()) return 0;
 #ifdef XINERAMA
diff --git a/src/Screen.hh b/src/Screen.hh
index ba550a7..d6007eb 100644
--- a/src/Screen.hh
+++ b/src/Screen.hh
@@ -317,6 +317,7 @@ public:
     int numHeads() const { return m_xinerama_num_heads; }
 
     void initXinerama();
+    void clearHeads();
 
     int getHead(int x, int y) const;
     int getHead(FbTk::FbWindow &win) const;
-- 
1.4.4.4

